@@ -1,7 +1,5 @@
+# -*- coding: utf-8 -*-
 # Pitivi video editor
-#
-#       pitivi/configure.py
-#
 # Copyright (c) 2005, Edward Hervey <bilboed@bilboed.com>
 #
 # This program is free software; you can redistribute it and/or
@@ -18,8 +16,8 @@
 # License along with this program; if not, write to the
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
-"""
-Utilities for getting the location of various directories.
+"""Utilities for getting the location of various directories.
+
 Enables identical use for installed and uninstalled versions.
 """
 
@@ -32,21 +30,13 @@ def _get_root_dir():
 
 
 def in_devel():
-    """
-    Returns whether the app is run from a git checkout.
-    """
-    rd = _get_root_dir()
-    return os.path.exists(os.path.join(rd, '.git'))
+    """Returns whether the app is run from a git checkout."""
+    return os.environ.get("PITIVI_DEVELOPMENT", "0") != "0"
 
-if "APPDIR" in os.environ:
-    basedir = os.environ["APPDIR"]
-    LIBDIR = os.path.join(basedir, 'usr', 'lib')
-    PKGDATADIR = os.path.join(basedir, 'usr', 'share', 'pitivi')
-else:
-    LIBDIR = '/usr/local/lib'
-    PKGDATADIR = '/usr/local/share/pitivi'
-
-VERSION = '0.95'
+LIBDIR = '/app/lib'
+PKGDATADIR = '/app/share/pitivi'
+GITVERSION = '0.96'
+VERSION = '0.96'
 APPNAME = 'Pitivi'
 APPURL = 'http://www.pitivi.org/'
 RELEASES_URL = 'http://pitivi.org/releases.txt'
@@ -71,25 +61,29 @@ def get_data_dir():
 
 
 def get_pixmap_dir():
-    """ Returns the directory for program-only pixmaps """
+    """Returns our directory with pixmaps."""
     return os.path.join(get_data_dir(), 'pixmaps')
 
 
 def get_ui_dir():
-    """ Returns the directory for GtkBuilder/Glade files """
+    """Returns our directory with Gtk.Builder/Glade files."""
     return os.path.join(get_data_dir(), 'ui')
 
 
 def get_renderpresets_dir():
-    """ Returns the directory for Render Presets files """
+    """Returns our directory with Render Presets files."""
     return os.path.join(get_data_dir(), 'renderpresets')
 
 
 def get_audiopresets_dir():
-    """ Returns the directory for Audio Presets files """
+    """Returns our directory with Audio Presets files."""
     return os.path.join(get_data_dir(), 'audiopresets')
 
 
 def get_videopresets_dir():
-    """ Returns the directory for Video Presets files """
+    """Returns our directory with Video Presets files."""
     return os.path.join(get_data_dir(), 'videopresets')
+
+def get_gstpresets_dir():
+    """Returns our directory with Gst Presets files."""
+    return os.path.join(get_data_dir(), 'gstpresets')
