@@ -1,7 +1,5 @@
+# -*- coding: utf-8 -*-
 # Pitivi video editor
-#
-#       pitivi/effects.py
-#
 # Copyright (c) 2010, Thibault Saunier <tsaunier@gnome.org>
 # Copyright (c) 2005, Edward Hervey <bilboed@bilboed.com>
 #
@@ -19,11 +17,7 @@
 # License along with this program; if not, write to the
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
-
-"""
-Effects global handling
- Note: Some effects are available through the frei0r
- library and the libavfilter0 library
+"""Effects categorization and management.
 
  There are different types of effects available:
   _ Simple Audio/Video Effects
@@ -34,26 +28,25 @@ Effects global handling
      that are too cumbersome to use as such
   _ Complex Audio/Video Effects
 """
-
 import os
 import re
+from gettext import gettext as _
 
+from gi.repository import Gdk
+from gi.repository import GdkPixbuf
 from gi.repository import GLib
 from gi.repository import Gst
 from gi.repository import Gtk
-from gi.repository import Gdk
 from gi.repository import Pango
-from gi.repository import GdkPixbuf
 
-from gettext import gettext as _
-
-from pitivi.configure import get_ui_dir, get_pixmap_dir
+from pitivi.configure import get_pixmap_dir
+from pitivi.configure import get_ui_dir
 from pitivi.settings import GlobalSettings
-
 from pitivi.utils.loggable import Loggable
-from pitivi.utils.ui import EFFECT_TARGET_ENTRY, SPACING
-
-from pitivi.utils.widgets import GstElementSettingsWidget, FractionWidget
+from pitivi.utils.ui import EFFECT_TARGET_ENTRY
+from pitivi.utils.ui import SPACING
+from pitivi.utils.widgets import FractionWidget
+from pitivi.utils.widgets import GstElementSettingsWidget
 
 
 (VIDEO_EFFECT, AUDIO_EFFECT) = list(range(1, 3))
@@ -130,19 +123,20 @@ VIDEO_EFFECTS_CATEGORIES = (
 
 BLACKLISTED_EFFECTS = ["colorconvert", "coglogoinsert", "festival",
                        "alphacolor", "cogcolorspace", "videodetect",
-                       "navigationtest", "videoanalyse"]
+                       "navigationtest", "videoanalyse", "volume"]
 
-# FIXME Check if this is still true with GES
-# We should unblacklist it when #650985 is solved
-BLACKLISTED_PLUGINS = ["ldaspa"]
+BLACKLISTED_PLUGINS = []
 
 ICON_WIDTH = 48 + 2 * 6  # 48 pixels, plus a margin on each side
 
 
-class EffectFactory(object):
-    """
-    Factories that applies an effect on a stream
+class EffectInfo(object):
+    """Info for displaying and using an effect.
+
+    Attributes:
+        effect_name (str): The bin_description identifying the effect.
     """
+
     def __init__(self, effect_name, media_type, categories,
                  human_name, description):
         object.__init__(self)
@@ -152,91 +146,98 @@ class EffectFactory(object):
         self.description = description
         self.human_name = human_name
 
+    @property
+    def icon(self):
+        pixdir = os.path.join(get_pixmap_dir(), "effects")
+        icon = None
+        try:
+            # We can afford to scale the images here, the impact is negligible
+            icon = GdkPixbuf.Pixbuf.new_from_file_at_size(
+                os.path.join(pixdir, self.effect_name + ".png"),
+                ICON_WIDTH, ICON_WIDTH)
+        # An empty except clause is bad, but "gi._glib.GError" is not helpful.
+        except:
+            icon = GdkPixbuf.Pixbuf.new_from_file(
+                os.path.join(pixdir, "defaultthumbnail.svg"))
+        return icon
+
 
 class EffectsManager(object):
+    """Keeps info about effects and their categories.
 
-    """
-    Groups effects.
+    Attributes:
+        video_effects (List[Gst.ElementFactory]): The available video effects.
+        audio_effects (List[Gst.ElementFactory]): The available audio effects.
     """
 
     def __init__(self):
         object.__init__(self)
-        self._pixdir = os.path.join(get_pixmap_dir(), "effects")
         self.video_effects = []
         self.audio_effects = []
-        self._effect_factories_dict = {}
-        self._setAllEffects()
+        self._effects = {}
+
+        uselessWords = [_("Video"), _("Audio"), _("Audio").lower(), _("effect")]
+        uselessRe = re.compile(" |".join(uselessWords))
 
-    def _setAllEffects(self):
-        """
-        go trough the list of element factories and
-        add them to the correct list filtering if necessary
-        """
         factories = Gst.Registry.get().get_feature_list(Gst.ElementFactory)
-        for element_factory in factories:
-            klass = element_factory.get_klass()
-            name = element_factory.get_name()
+        longnames = set()
+        duplicate_longnames = set()
+        for factory in factories:
+            longname = factory.get_longname()
+            if longname in longnames:
+                duplicate_longnames.add(longname)
+            else:
+                longnames.add(longname)
+        for factory in factories:
+            klass = factory.get_klass()
+            name = factory.get_name()
+            if ("Effect" not in klass or
+                    any(black in name for black in BLACKLISTED_PLUGINS)):
+                continue
 
-            if ("Effect" in klass and name not in BLACKLISTED_EFFECTS and not
-                    [bplug for bplug in BLACKLISTED_PLUGINS if bplug in name]):
-                media_type = None
+            media_type = None
+            if "Audio" in klass:
+                self.audio_effects.append(factory)
+                media_type = AUDIO_EFFECT
+            elif "Video" in klass:
+                self.video_effects.append(factory)
+                media_type = VIDEO_EFFECT
+            if not media_type:
+                HIDDEN_EFFECTS.append(name)
+                continue
 
-                if "Audio" in klass:
-                    self.audio_effects.append(element_factory)
-                    media_type = AUDIO_EFFECT
-                elif "Video" in klass:
-                    self.video_effects.append(element_factory)
-                    media_type = VIDEO_EFFECT
+            longname = factory.get_longname()
+            if longname in duplicate_longnames:
+                # Workaround https://bugzilla.gnome.org/show_bug.cgi?id=760566
+                # Add name which identifies the element and is unique.
+                longname = "%s %s" % (longname, name)
+            human_name = uselessRe.sub("", longname).title()
+            effect = EffectInfo(name,
+                                media_type,
+                                categories=self._getEffectCategories(name),
+                                human_name=human_name,
+                                description=factory.get_description())
+            self._effects[name] = effect
 
-                if not media_type:
-                    HIDDEN_EFFECTS.append(name)
-                    continue
+    def getInfo(self, name):
+        """Gets the info for an effect which can be applied.
 
-                effect = EffectFactory(name,
-                                       media_type,
-                                       categories=self._getEffectCategories(
-                                           name),
-                                       human_name=self._getEffectName(
-                                           element_factory),
-                                       description=self._getEffectDescripton(element_factory))
-                self._addEffectToDic(name, effect)
+        Args:
+            name (str): The bin_description identifying the effect.
 
-    def getAllAudioEffects(self):
+        Returns:
+            EffectInfo: The info corresponding to the name, or None.
         """
-        @return: the list of available audio effects elements
-        """
-        return self.audio_effects
-
-    def getAllVideoEffects(self):
-        """
-        @return: the list of available video effects elements
-        """
-        return self.video_effects
-
-    def _addEffectToDic(self, name, factory):
-        self._effect_factories_dict[name] = factory
-
-    def getFactoryFromName(self, name):
-        """
-        @param name: The bin_description of the effect.
-        @type name: C{str}
-        @return: The l{EffectFactory} corresponding to the name or None
-        """
-        return self._effect_factories_dict.get(name)
-
-    def _getEffectDescripton(self, element_factory):
-        """
-        @param element_factory: The element factory
-        @type element_factory: L{Gst.ElementFactory}
-        @return: A human description C{str} for the effect
-        """
-        return element_factory.get_description()
+        return self._effects.get(name)
 
     def _getEffectCategories(self, effect_name):
-        """
-        @param effect_name: the name of the effect for wich we want the category
-        @type effect_name: L{str}
-        @return: A C{list} of name C{str} of categories corresponding the effect
+        """Gets the categories to which the specified effect belongs.
+
+        Args:
+            effect_name (str): The bin_description identifying the effect.
+
+        Returns:
+            List[str]: The categories which contain the effect.
         """
         categories = []
         for category_name, effects in AUDIO_EFFECTS_CATEGORIES:
@@ -246,39 +247,20 @@ class EffectsManager(object):
             if effect_name in effects:
                 categories.append(category_name)
         if not categories:
-            uncategorized = _("Uncategorized")
-            categories.append(uncategorized)
+            categories.append(_("Uncategorized"))
         categories.insert(0, _("All effects"))
         return categories
 
-    def _getEffectName(self, element_factory):
-        """
-        @param element_factory: The element factory
-        @type element_factory: L{Gst.ElementFactory}
-        @return: A human readable name C{str} for the effect
-        """
-        video = _("Video")
-        audio = _("Audio")
-        effect = _("effect")
-        uselessWords = re.compile(" |".join([video, audio, audio.lower(), effect]))
-        return uselessWords.sub("", element_factory.get_longname()).title()
-
-    def getVideoCategories(self):
-        """
-        Get all video effect categories names.
-        """
+    @property
+    def video_categories(self):
+        """Gets all video effect categories names."""
         return EffectsManager._getCategoriesNames(VIDEO_EFFECTS_CATEGORIES)
 
-    video_categories = property(getVideoCategories)
-
-    def getAudioCategories(self):
-        """
-        Get all audio effect categories names.
-        """
+    @property
+    def audio_categories(self):
+        """Gets all audio effect categories names."""
         return EffectsManager._getCategoriesNames(AUDIO_EFFECTS_CATEGORIES)
 
-    audio_categories = property(getAudioCategories)
-
     @staticmethod
     def _getCategoriesNames(categories):
         ret = [category_name for category_name, unused_effects in categories]
@@ -289,22 +271,13 @@ class EffectsManager(object):
             ret.append(_("Uncategorized"))
         return ret
 
-    def getEffectIcon(self, effect_name):
-        icon = None
-        try:
-            # We can afford to scale the images here, the impact is negligible
-            icon = GdkPixbuf.Pixbuf.new_from_file_at_size(
-                os.path.join(self._pixdir, effect_name + ".png"),
-                ICON_WIDTH, ICON_WIDTH)
-        # An empty except clause is bad, but "gi._glib.GError" is not helpful.
-        except:
-            icon = GdkPixbuf.Pixbuf.new_from_file(
-                os.path.join(self._pixdir, "defaultthumbnail.svg"))
-        return icon
-
 
 # ----------------------- UI classes to manage effects -------------------------#
-HIDDEN_EFFECTS = ["frei0r-filter-scale0tilt"]
+
+HIDDEN_EFFECTS = [
+    # Overlaying an image onto a video stream can already be done.
+    "gdkpixbufoverlay",
+    "frei0r-filter-scale0tilt"]
 
 GlobalSettings.addConfigSection('effect-library')
 
@@ -312,16 +285,14 @@ GlobalSettings.addConfigSection('effect-library')
  COL_DESC_TEXT,
  COL_EFFECT_TYPE,
  COL_EFFECT_CATEGORIES,
- COL_FACTORY,
  COL_ELEMENT_NAME,
- COL_ICON) = list(range(7))
+ COL_ICON) = list(range(6))
 
 
 class EffectListWidget(Gtk.Box, Loggable):
+    """Widget for listing effects."""
 
-    """ Widget for listing effects """
-
-    def __init__(self, instance, unused_uiman):
+    def __init__(self, instance):
         Gtk.Box.__init__(self)
         Loggable.__init__(self)
 
@@ -343,7 +314,7 @@ class EffectListWidget(Gtk.Box, Loggable):
 
         # Store
         self.storemodel = Gtk.ListStore(
-            str, str, int, object, object, str, GdkPixbuf.Pixbuf)
+            str, str, int, object, str, GdkPixbuf.Pixbuf)
 
         self.view = Gtk.TreeView(model=self.storemodel)
         self.view.props.headers_visible = False
@@ -373,11 +344,14 @@ class EffectListWidget(Gtk.Box, Loggable):
         text_cell.set_property("ellipsize", Pango.EllipsizeMode.END)
         text_col.pack_start(text_cell, True)
         text_col.set_cell_data_func(
-            text_cell, self.view_description_cell_data_func, None)
+            text_cell, self.viewDescriptionCellDataFunc, None)
 
         self.view.append_column(icon_col)
         self.view.append_column(text_col)
 
+        self.view.connect("query-tooltip", self._treeViewQueryTooltipCb)
+        self.view.props.has_tooltip = True
+
         # Make the treeview a drag source which provides effects.
         self.view.enable_model_drag_source(
             Gdk.ModifierType.BUTTON1_MASK, [EFFECT_TARGET_ENTRY], Gdk.DragAction.COPY)
@@ -404,30 +378,41 @@ class EffectListWidget(Gtk.Box, Loggable):
         scrollwin.show_all()
         toolbar.show_all()
 
-    @staticmethod
-    def view_description_cell_data_func(unused_column, cell, model, iter_, unused_data):
-        name, desc = model.get(iter_, COL_NAME_TEXT, COL_DESC_TEXT)
+    def _treeViewQueryTooltipCb(self, view, x, y, keyboard_mode, tooltip):
+        is_row, x, y, model, path, tree_iter = view.get_tooltip_context(
+            x, y, keyboard_mode)
+        if not is_row:
+            return False
+
+        view.set_tooltip_row(tooltip, path)
+        tooltip.set_markup(self.formatDescription(model, tree_iter))
+        return True
+
+    def viewDescriptionCellDataFunc(self, unused_column, cell, model, iter_, unused_data):
+        cell.props.markup = self.formatDescription(model, iter_)
+
+    def formatDescription(self, model, iter_):
+        name, element_name, desc = model.get(iter_, COL_NAME_TEXT, COL_ELEMENT_NAME, COL_DESC_TEXT)
         escape = GLib.markup_escape_text
-        cell.props.markup = "<b>%s</b>\n%s" % (escape(name),
-                                               escape(desc),)
+        return "<b>%s</b>\n%s" % (escape(name), escape(desc))
 
     def _loadAvailableEffectsCb(self):
-        self._addFactories(self.app.effects.getAllVideoEffects(), VIDEO_EFFECT)
-        self._addFactories(self.app.effects.getAllAudioEffects(), AUDIO_EFFECT)
+        self._addFactories(self.app.effects.video_effects, VIDEO_EFFECT)
+        self._addFactories(self.app.effects.audio_effects, AUDIO_EFFECT)
         return False
 
     def _addFactories(self, elements, effectType):
         for element in elements:
             name = element.get_name()
-            if name not in HIDDEN_EFFECTS:
-                effect_factory = self.app.effects.getFactoryFromName(name)
-                self.storemodel.append([effect_factory.human_name,
-                                        effect_factory.description,
-                                        effectType,
-                                        effect_factory.categories,
-                                        effect_factory,
-                                        name,
-                                        self.app.effects.getEffectIcon(name)])
+            if name in HIDDEN_EFFECTS:
+                continue
+            effect_info = self.app.effects.getInfo(name)
+            self.storemodel.append([effect_info.human_name,
+                                    effect_info.description,
+                                    effectType,
+                                    effect_info.categories,
+                                    name,
+                                    effect_info.icon])
         self.storemodel.set_sort_column_id(
             COL_NAME_TEXT, Gtk.SortType.ASCENDING)
 
@@ -447,8 +432,8 @@ class EffectListWidget(Gtk.Box, Loggable):
         self.categoriesWidget.set_active(0)
 
     def _dndDragDataGetCb(self, unused_view, drag_context, selection_data, unused_info, unused_timestamp):
-        factory_name = bytes(self.getSelectedEffectFactoryName(), "UTF-8")
-        selection_data.set(drag_context.list_targets()[0], 0, factory_name)
+        data = bytes(self.getSelectedEffect(), "UTF-8")
+        selection_data.set(drag_context.list_targets()[0], 0, data)
         return True
 
     def _rowUnderMouseSelected(self, view, event):
@@ -461,10 +446,7 @@ class EffectListWidget(Gtk.Box, Loggable):
         return False
 
     def _enterPressEventCb(self, unused_view, unused_event=None):
-        factory_name = self.getSelectedEffectFactoryName()
-        if factory_name is not None:
-            self.app.gui.clipconfig.effect_expander.addEffectToCurrentSelection(
-                factory_name)
+        self._addSelectedEffect()
 
     def _buttonPressEventCb(self, view, event):
         chain_up = True
@@ -472,22 +454,25 @@ class EffectListWidget(Gtk.Box, Loggable):
         if event.button == 3:
             chain_up = False
         elif event.type == getattr(Gdk.EventType, '2BUTTON_PRESS'):
-            factory_name = self.getSelectedEffectFactoryName()
-            if factory_name is not None:
-                self.app.gui.clipconfig.effect_expander.addEffectToCurrentSelection(
-                    factory_name)
+            self._addSelectedEffect()
         else:
             chain_up = not self._rowUnderMouseSelected(view, event)
 
         if chain_up:
             self._draggedItems = None
         else:
-            self._draggedItems = self.getSelectedEffectFactoryName()
+            self._draggedItems = self.getSelectedEffect()
 
         Gtk.TreeView.do_button_press_event(view, event)
         return True
 
-    def getSelectedEffectFactoryName(self):
+    def _addSelectedEffect(self):
+        effect = self.getSelectedEffect()
+        if not effect:
+            return
+        self.app.gui.clipconfig.effect_expander.addEffectToCurrentSelection(effect)
+
+    def getSelectedEffect(self):
         if self._draggedItems:
             return self._draggedItems
         model, rows = self.view.get_selection().get_selected_rows()
@@ -495,8 +480,8 @@ class EffectListWidget(Gtk.Box, Loggable):
         return self.storemodel[path][COL_ELEMENT_NAME]
 
     def _toggleViewTypeCb(self, widget):
-        """
-        Handle the switching of the view mode between video and audio.
+        """Switches the view mode between video and audio.
+
         This makes the two togglebuttons behave like a group of radiobuttons.
         """
         if widget is self.video_togglebutton:
@@ -509,7 +494,6 @@ class EffectListWidget(Gtk.Box, Loggable):
             self._effectType = VIDEO_EFFECT
         else:
             self._effectType = AUDIO_EFFECT
-
         self.populate_categories_widget()
         self.modelFilter.refilter()
 
@@ -523,67 +507,59 @@ class EffectListWidget(Gtk.Box, Loggable):
         entry.set_text("")
 
     def _setRowVisible(self, model, iter, unused_data):
-        if self._effectType == model.get_value(iter, COL_EFFECT_TYPE):
-            if model.get_value(iter, COL_EFFECT_CATEGORIES) is None:
-                return False
-            if self.categoriesWidget.get_active_text() in model.get_value(iter, COL_EFFECT_CATEGORIES):
-                text = self.searchEntry.get_text().lower()
-                return text in model.get_value(iter, COL_DESC_TEXT).lower() or\
-                    text in model.get_value(iter, COL_NAME_TEXT).lower()
-            else:
-                return False
-        else:
+        if not self._effectType == model.get_value(iter, COL_EFFECT_TYPE):
             return False
+        if model.get_value(iter, COL_EFFECT_CATEGORIES) is None:
+            return False
+        if self.categoriesWidget.get_active_text() not in model.get_value(iter, COL_EFFECT_CATEGORIES):
+            return False
+        text = self.searchEntry.get_text().lower()
+        return text in model.get_value(iter, COL_DESC_TEXT).lower() or\
+            text in model.get_value(iter, COL_NAME_TEXT).lower()
 
 
-PROPS_TO_IGNORE = ['name', 'qos', 'silent', 'message']
+PROPS_TO_IGNORE = ['name', 'qos', 'silent', 'message', 'parent']
 
 
 class EffectsPropertiesManager:
+    """Provides and caches UIs for editing effects.
 
-    """
-    @type app: L{Pitivi}
+    Attributes:
+        app (Pitivi): The app.
     """
 
     def __init__(self, app):
         self.cache_dict = {}
-        self._current_effect_setting_ui = None
         self._current_element_values = {}
-        self.action_log = app.action_log
         self.app = app
 
     def getEffectConfigurationUI(self, effect):
-        """Permit to get a configuration GUI for the effect
+        """Gets a configuration UI element for the effect.
 
-        @param effect: The effect for which we want the configuration UI
-        @type effect: C{Gst.Element}
+        Args:
+            effect (Gst.Element): The effect for which we want the UI.
+
+        Returns:
+            GstElementSettingsWidget: A container for configuring the effect.
         """
         if effect not in self.cache_dict:
             # Here we should handle special effects configuration UI
-            effect_settings_widget = GstElementSettingsWidget()
-            effect_settings_widget.setElement(effect, ignore=PROPS_TO_IGNORE,
-                                              default_btn=True, use_element_props=True)
-            scrolled_window = Gtk.ScrolledWindow()
-            scrolled_window.add_with_viewport(effect_settings_widget)
-            scrolled_window.set_policy(Gtk.PolicyType.AUTOMATIC,
-                                       Gtk.PolicyType.AUTOMATIC)
-            self.cache_dict[effect] = scrolled_window
-            self._connectAllWidgetCallbacks(effect_settings_widget, effect)
-            self._postConfiguration(effect, effect_settings_widget)
+            effect_widget = GstElementSettingsWidget()
+            effect_widget.setElement(effect, ignore=PROPS_TO_IGNORE,
+                                     with_reset_button=True)
+            self.cache_dict[effect] = effect_widget
+            self._connectAllWidgetCallbacks(effect_widget, effect)
+            self._postConfiguration(effect, effect_widget)
 
-        self._current_effect_setting_ui = self._getUiToSetEffect(effect)
-        element = self._current_effect_setting_ui.element
-        for prop in element.list_children_properties():
-            self._current_element_values[
-                prop.name] = element.get_child_property(prop.name)
+        for prop in effect.list_children_properties():
+            value = effect.get_child_property(prop.name)
+            self._current_element_values[prop.name] = value
 
         return self.cache_dict[effect]
 
     def cleanCache(self, effect):
         if effect in self.cache_dict:
-            conf_ui = self.cache_dict.get(effect)
-            self.cache_dict.pop(effect)
-            return conf_ui
+            return self.cache_dict.pop(effect)
 
     def _postConfiguration(self, effect, effect_set_ui):
         if 'aspectratiocrop' in effect.get_property("bin-description"):
@@ -591,34 +567,26 @@ class EffectsPropertiesManager:
                 if isinstance(widget, FractionWidget):
                     widget.addPresets(["4:3", "5:4", "9:3", "16:9", "16:10"])
 
-    def _getUiToSetEffect(self, effect):
-        """ Permit to get the widget to set the effect and not its container """
-        if type(self.cache_dict[effect]) is Gtk.ScrolledWindow:
-            effect_set_ui = self.cache_dict[
-                effect].get_children()[0].get_children()[0]
-        else:
-            effect_set_ui = self.cache_dict[effect]
-        return effect_set_ui
-
-    def _connectAllWidgetCallbacks(self, effect_settings_widget, unused_effect):
+    def _connectAllWidgetCallbacks(self, effect_settings_widget, effect):
         for prop, widget in effect_settings_widget.properties.items():
-            widget.connectValueChanged(self._onValueChangedCb, widget, prop)
+            widget.connectValueChanged(self._onValueChangedCb, widget, prop, effect)
 
-    def _onSetDefaultCb(self, unused_widget, dynamic):
-        dynamic.setWidgetToDefault()
+    def _onSetDefaultCb(self, unused_widget, effect_widget):
+        effect_widget.setWidgetToDefault()
 
-    def _onValueChangedCb(self, unused_widget, dynamic, prop):
-        value = dynamic.getWidgetValue()
+    def _onValueChangedCb(self, unused_widget, effect_widget, prop, effect):
+        value = effect_widget.getWidgetValue()
 
         # FIXME Workaround in order to make aspectratiocrop working
         if isinstance(value, Gst.Fraction):
             value = Gst.Fraction(int(value.num), int(value.denom))
 
         if value != self._current_element_values.get(prop.name):
-            self.action_log.begin("Effect property change")
-            self._current_effect_setting_ui.element.set_child_property(
-                prop.name, value)
-            self.action_log.commit()
+            from pitivi.undo.timeline import CommitTimelineFinalizingAction
 
-            self.app.project_manager.current_project.pipeline.flushSeek()
+            pipeline = self.app.project_manager.current_project.pipeline
+            with self.app.action_log.started("Effect property change",
+                                             CommitTimelineFinalizingAction(pipeline)):
+                effect.set_child_property(prop.name, value)
+            pipeline.commit_timeline()
             self._current_element_values[prop.name] = value
