@@ -1,8 +1,5 @@
 # -*- coding: utf-8 -*-
 # Pitivi video editor
-#
-#       pitivi/timeline/ruler.py
-#
 # Copyright (c) 2006, Edward Hervey <bilboed@bilboed.com>
 # Copyright (c) 2014, Alex Băluț <alexandru.balut@gmail.com>
 #
@@ -20,32 +17,58 @@
 # License along with this program; if not, write to the
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
-
-import cairo
-
-from gi.repository import Gtk
-from gi.repository import Gdk
-from gi.repository import Gst
-from gi.repository import GLib
-from gi.repository import GObject
-
 from gettext import gettext as _
 
-from pitivi.utils.pipeline import Seeker
-from pitivi.utils.timeline import Zoomable
+import cairo
+from gi.repository import Gdk
+from gi.repository import Gst
+from gi.repository import Gtk
+
 from pitivi.utils.loggable import Loggable
-from pitivi.utils.ui import NORMAL_FONT, PLAYHEAD_WIDTH, set_cairo_color, time_to_string, beautify_length
+from pitivi.utils.timeline import Zoomable
+from pitivi.utils.ui import beautify_length
+from pitivi.utils.ui import gtk_style_context_get_color
+from pitivi.utils.ui import NORMAL_FONT
+from pitivi.utils.ui import PLAYHEAD_COLOR
+from pitivi.utils.ui import PLAYHEAD_WIDTH
+from pitivi.utils.ui import set_cairo_color
+from pitivi.utils.ui import time_to_string
 
 
-HEIGHT = 25
+# Tuples of:
+# - an interval lengths in seconds for which a timestamp will be displayed
+# - how the ticks should be displayed for this interval:
+#   (count per interval, height ratio) tuples.
+SCALES = (
+    (0.1, ((0.1, 1.0), (0.05, .5), (0.01, .25))),
+    (0.2, ((0.2, 1.0), (0.1, .5), (0.05, .25))),
+    (0.5, ((0.5, 1.0), (0.1, .25))),
 
-# A series of valid interval lengths in seconds.
-SCALES = (0.5, 1, 2, 5, 10, 15, 30, 60, 120, 300, 600, 3600)
+    # 1 second.
+    (1, ((1, 1.0), (0.5, .5), (0.1, .25))),
+    (2, ((2, 1.0), (1, .5), (0.5, .25))),
+    (5, ((5, 1.0), (1, .25))),
+    (10, ((10, 1.0), (5, .5), (1, .25))),
+    (20, ((20, 1.0), (10, .5), (1, .25))),
+    (30, ((30, 1.0), (10, .5), (1, .25))),
+
+    # 1 minute.
+    (60, ((60, 1.0), (30, .5), (15, .25))),
+    # 2 minutes.
+    (120, ((120, 1.0), (60, .5), (30, .25))),
+    # 5 minutes.
+    (300, ((300, 1.0), (60, .25))),
+    # 10 minutes.
+    (600, ((600, 1.0), (300, .5), (60, .25))),
+    # 30 minutes.
+    (1800, ((1800, 1.0), (900, .5), (450, .25))),
+
+    # 1 hour.
+    (3600, ((3600, 1.0), (1800, .75), (900, .5))),
+)
 
 # The minimum distance between adjacent ticks.
-MIN_TICK_SPACING_PIXELS = 3
-# (count per interval, height ratio) tuples determining how the ticks appear.
-TICK_TYPES = ((1, 1.0), (2, 0.5), (10, .25))
+MIN_TICK_SPACING_PIXELS = 6
 
 # For displaying the times a bit to the right.
 TIMES_LEFT_MARGIN_PIXELS = 3
@@ -60,22 +83,16 @@ SMALL_FONT_SIZE = 11
 
 
 class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
-
-    """
-    Widget for displaying the ruler.
+    """Widget for displaying the ruler.
 
     Displays a series of consecutive intervals. For each interval its beginning
     time is shown. If zoomed in enough, shows the frames in alternate colors.
-    """
 
-    __gsignals__ = {
-        "button-press-event": "override",
-        "button-release-event": "override",
-        "motion-notify-event": "override",
-        "scroll-event": "override",
-        "seek": (GObject.SignalFlags.RUN_LAST, None,
-                 [GObject.TYPE_UINT64])
-    }
+    Attributes:
+        timeline (TimelineContainer): The timeline container used to handle
+            scroll events.
+        _pipeline (Pipeline): The pipeline of the project.
+    """
 
     def __init__(self, timeline, hadj):
         Gtk.DrawingArea.__init__(self)
@@ -83,15 +100,8 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         Loggable.__init__(self)
         self.log("Creating new ScaleRuler")
 
-        # Allows stealing focus from other GTK widgets, prevent accidents:
-        self.props.can_focus = True
-        self.connect("focus-in-event", self._focusInCb)
-        self.connect("focus-out-event", self._focusOutCb)
-
         self.timeline = timeline
-        self._background_color = timeline.get_style_context().lookup_color(
-            'theme_bg_color')[1]
-        self._seeker = Seeker()
+        self._pipeline = None
         self.hadj = hadj
         hadj.connect("value-changed", self._hadjValueChangedCb)
         self.add_events(Gdk.EventMask.POINTER_MOTION_MASK |
@@ -103,38 +113,13 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         # all values are in pixels
         self.pixbuf_offset = 0
         self.pixbuf_offset_painted = 0
-        # This is the number of width we allocate for the pixbuf
-        self.pixbuf_multiples = 4
 
         self.position = 0  # In nanoseconds
-        self.pressed = False
         self.frame_rate = Gst.Fraction(1 / 1)
         self.ns_per_frame = float(1 / self.frame_rate) * Gst.SECOND
-        self.connect('draw', self.drawCb)
-        self.connect('configure-event', self.configureEventCb)
-        self.callback_id = None
-        self.set_size_request(0, HEIGHT)
-
-        style = self.get_style_context()
-        color_normal = style.get_color(Gtk.StateFlags.NORMAL)
-        color_insensitive = style.get_color(Gtk.StateFlags.INSENSITIVE)
-        self._color_normal = color_normal
-        self._color_dimmed = Gdk.RGBA(
-            *[(x * 3 + y * 2) / 5
-              for x, y in ((color_normal.red, color_insensitive.red),
-                           (color_normal.green, color_insensitive.green),
-                           (color_normal.blue, color_insensitive.blue))])
 
         self.scales = SCALES
 
-    def _focusInCb(self, unused_widget, unused_arg):
-        self.log("Ruler has grabbed focus")
-        self.timeline.setActionsSensitivity(True)
-
-    def _focusOutCb(self, unused_widget, unused_arg):
-        self.log("Ruler has lost focus")
-        self.timeline.setActionsSensitivity(False)
-
     def _hadjValueChangedCb(self, unused_arg):
         self.pixbuf_offset = self.hadj.get_value()
         self.queue_draw()
@@ -147,16 +132,18 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
 # Timeline position changed method
 
     def setPipeline(self, pipeline):
-        pipeline.connect('position', self.timelinePositionCb)
+        self._pipeline = pipeline
+        self._pipeline.connect('position', self.timelinePositionCb)
 
     def timelinePositionCb(self, unused_pipeline, position):
         self.position = position
         self.queue_draw()
 
 # Gtk.Widget overrides
-    def configureEventCb(self, widget, unused_event, unused_data=None):
-        width = widget.get_allocated_width()
-        height = widget.get_allocated_height()
+
+    def do_configure_event(self, unused_event):
+        width = self.get_allocated_width()
+        height = self.get_allocated_height()
         self.debug("Configuring, height %d, width %d", width, height)
 
         # Destroy previous buffer
@@ -167,9 +154,23 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         # Create a new buffer
         self.pixbuf = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
 
+        context = self.app.gui.get_style_context()
+
+        color_normal = gtk_style_context_get_color(context, Gtk.StateFlags.NORMAL)
+        color_insensitive = gtk_style_context_get_color(context, Gtk.StateFlags.BACKDROP)
+        self._color_normal = color_normal
+        self._color_dimmed = Gdk.RGBA(
+            *[(x * 3 + y * 2) / 5
+              for x, y in ((color_normal.red, color_insensitive.red),
+                           (color_normal.green, color_insensitive.green),
+                           (color_normal.blue, color_insensitive.blue))])
+
+        # Two colors with high contrast.
+        self._color_frame = gtk_style_context_get_color(context, Gtk.StateFlags.LINK)
+
         return False
 
-    def drawCb(self, unused_widget, context):
+    def do_draw(self, context):
         if self.pixbuf is None:
             self.info('No buffer to paint')
             return False
@@ -189,23 +190,33 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         return False
 
     def do_button_press_event(self, event):
+        if not self._pipeline:
+            return False
+
         self.debug("button pressed at x:%d", event.x)
-        self.pressed = True
-        position = self.pixelToNs(event.x + self.pixbuf_offset)
-        self._seeker.seek(position, on_idle=True)
-        return True
+        button = event.button
+        if button == 3 or (button == 1 and self.app.settings.leftClickAlsoSeeks):
+            position = self.pixelToNs(event.x + self.pixbuf_offset)
+            self._pipeline.simple_seek(position)
+        return False
 
     def do_button_release_event(self, event):
         self.debug("button released at x:%d", event.x)
-        self.grab_focus()  # Prevent other widgets from being confused
-        self.pressed = False
+        self.app.gui.focusTimeline()
         return False
 
     def do_motion_notify_event(self, event):
+        if not self._pipeline:
+            return False
+
         position = self.pixelToNs(event.x + self.pixbuf_offset)
-        if self.pressed:
+
+        seek_mask = Gdk.ModifierType.BUTTON3_MASK
+        if self.app.settings.leftClickAlsoSeeks:
+            seek_mask |= Gdk.ModifierType.BUTTON1_MASK
+        if event.state & seek_mask:
             self.debug("motion at event.x %d", event.x)
-            self._seeker.seek(position, on_idle=True)
+            self._pipeline.simple_seek(position)
 
         human_time = beautify_length(position)
         cur_frame = int(position / self.ns_per_frame) + 1
@@ -213,71 +224,49 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         return False
 
     def do_scroll_event(self, event):
-        if event.scroll.state & Gdk.ModifierType.CONTROL_MASK:
-            # Control + scroll = zoom
-            if event.scroll.direction == Gdk.ScrollDirection.UP:
-                Zoomable.zoomIn()
-            elif event.scroll.direction == Gdk.ScrollDirection.DOWN:
-                Zoomable.zoomOut()
-        else:
-            # No modifier key held down, just scroll
-            if event.scroll.direction in (Gdk.ScrollDirection.UP,
-                                          Gdk.ScrollDirection.LEFT):
-                self.timeline.scroll_left()
-            elif event.scroll.direction in (Gdk.ScrollDirection.DOWN,
-                                            Gdk.ScrollDirection.RIGHT):
-                self.timeline.scroll_right()
+        self.timeline.timeline.do_scroll_event(event)
 
     def setProjectFrameRate(self, rate):
-        """
-        Set the lowest scale based on project framerate
-        """
+        """Sets the lowest scale based on the specified project framerate."""
         self.frame_rate = rate
-        self.ns_per_frame = float(1 / self.frame_rate) * Gst.SECOND
-        self.scales = (float(2 / rate), float(
-            5 / rate), float(10 / rate)) + SCALES
+        self.ns_per_frame = float(Gst.SECOND / self.frame_rate)
+        self.scales = (2 / rate, 5 / rate, 10 / rate) + SCALES
 
 # Drawing methods
 
     def drawBackground(self, context):
-        style = self.get_style_context()
-        set_cairo_color(context, self._background_color)
         width = context.get_target().get_width()
         height = context.get_target().get_height()
-        context.rectangle(0, 0, width, height)
-        context.fill()
-        offset = int(self.nsToPixel(Gst.CLOCK_TIME_NONE)) - self.pixbuf_offset
-        if offset > 0:
-            set_cairo_color(
-                context, style.get_background_color(Gtk.StateFlags.ACTIVE))
-            context.rectangle(
-                0, 0, int(offset), height)
-            context.fill()
+        style_context = self.app.gui.get_style_context()
+        Gtk.render_background(style_context, context, 0, 0, width, height)
 
     def drawRuler(self, context):
         context.set_font_face(NORMAL_FONT)
         context.set_font_size(NORMAL_FONT_SIZE)
 
-        spacing, scale = self._getSpacing(context)
+        spacing, interval_seconds, ticks = self._getSpacing(context)
         offset = self.pixbuf_offset % spacing
         self.drawFrameBoundaries(context)
-        self.drawTicks(context, offset, spacing)
-        self.drawTimes(context, offset, spacing, scale)
+        self.drawTicks(context, offset, spacing, interval_seconds, ticks)
+        self.drawTimes(context, offset, spacing, interval_seconds)
 
     def _getSpacing(self, context):
-        textwidth = context.text_extents(time_to_string(0))[2]
+        # The longest timestamp we display is 0:00:00 because
+        # when we display millis, they are displayed by themselves.
+        min_interval_width = context.text_extents("0:00:00")[2] * 1.3
         zoom = Zoomable.zoomratio
-        for scale in self.scales:
-            spacing = scale * zoom
-            if spacing >= textwidth * 1.5:
-                return spacing, scale
+        for interval_seconds, ticks in SCALES:
+            interval_width = interval_seconds * zoom
+            if interval_width >= min_interval_width:
+                return interval_width, interval_seconds, ticks
         raise Exception(
             "Failed to find an interval size for textwidth:%s, zoomratio:%s" %
-            (textwidth, Zoomable.zoomratio))
+            (min_interval_width, Zoomable.zoomratio))
 
-    def drawTicks(self, context, offset, spacing):
-        for count_per_interval, height_ratio in TICK_TYPES:
-            space = float(spacing) / count_per_interval
+    def drawTicks(self, context, offset, spacing, interval_seconds, ticks):
+        for tick_interval, height_ratio in ticks:
+            count_per_interval = interval_seconds / tick_interval
+            space = spacing / count_per_interval
             if space < MIN_TICK_SPACING_PIXELS:
                 break
             paintpos = 0.5 - offset
@@ -297,20 +286,17 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         context.close_path()
         context.stroke()
 
-    def drawTimes(self, context, offset, spacing, scale):
+    def drawTimes(self, context, offset, spacing, interval_seconds):
         # figure out what the optimal offset is
-        interval = int(Gst.SECOND * scale)
+        interval = int(Gst.SECOND * interval_seconds)
         current_time = self.pixelToNs(self.pixbuf_offset)
         paintpos = TIMES_LEFT_MARGIN_PIXELS
         if offset > 0:
             current_time = current_time - (current_time % interval) + interval
             paintpos += spacing - offset
 
-        state = Gtk.StateFlags.NORMAL
-        style = self.get_style_context()
-        set_cairo_color(context, style.get_color(state))
+        set_cairo_color(context, self._color_normal)
         y_bearing = context.text_extents("0")[1]
-        millis = scale < 1
 
         def split(x):
             # Seven elements: h : mm : ss . mmm
@@ -323,6 +309,7 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         while paintpos < width:
             context.move_to(int(paintpos), 1 - y_bearing)
             current = split(time_to_string(int(current_time)))
+            millis = current_time % Gst.SECOND > 0
             self._drawTime(context, current, previous, millis)
             previous = current
             paintpos += spacing
@@ -332,15 +319,22 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         hour = int(current[0])
         for index, (element, previous_element) in enumerate(zip(current, previous)):
             if index <= 1 and not hour:
+                # Don't draw hour if 0.
                 continue
-            if index >= 5 and not millis:
-                break
+            if millis:
+                # Draw only the millis.
+                if index < 5:
+                    continue
+            else:
+                # Don't draw the millis.
+                if index == 5:
+                    break
             if element == previous_element:
                 color = self._color_dimmed
             else:
                 color = self._color_normal
             set_cairo_color(context, color)
-            # Display the millis with a smaller font
+            # Display the millis with a smaller font.
             small = index >= 5
             if small:
                 context.set_font_size(SMALL_FONT_SIZE)
@@ -349,10 +343,12 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
                 context.set_font_size(NORMAL_FONT_SIZE)
 
     def drawFrameBoundaries(self, context):
-        """
-        Draw the alternating rectangles that represent the project frames at
-        high zoom levels. These are based on the framerate set in the project
-        settings, not the actual frames on a video codec level.
+        """Draws the alternating rectangles that represent the project frames.
+
+        These are drawn only at high zoom levels.
+
+        These are based on the project's framerate settings, not the actual
+        frames on the assets.
         """
         frame_width = self.nsToPixel(self.ns_per_frame)
         if not frame_width >= FRAME_MIN_WIDTH_PIXELS:
@@ -361,11 +357,6 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         offset = self.pixbuf_offset % frame_width
         height = context.get_target().get_height()
         y = int(height - FRAME_HEIGHT_PIXELS)
-        # INSENSITIVE is a dark shade of gray, but lacks contrast
-        # SELECTED will be bright blue and more visible to represent frames
-        style = self.get_style_context()
-        states = [style.get_background_color(Gtk.StateFlags.ACTIVE),
-                  style.get_background_color(Gtk.StateFlags.SELECTED)]
 
         frame_num = int(
             self.pixelToNs(self.pixbuf_offset) * float(self.frame_rate) / Gst.SECOND)
@@ -374,18 +365,39 @@ class ScaleRuler(Gtk.DrawingArea, Zoomable, Loggable):
         while paintpos < max_pos:
             paintpos = self.nsToPixel(
                 1 / float(self.frame_rate) * Gst.SECOND * frame_num)
-            set_cairo_color(context, states[(frame_num + 1) % 2])
-            context.rectangle(
-                0.5 + paintpos - self.pixbuf_offset, y, frame_width, height)
-            context.fill()
+            if frame_num % 2:
+                set_cairo_color(context, self._color_frame)
+                context.rectangle(
+                    0.5 + paintpos - self.pixbuf_offset, y, frame_width, height)
+                context.fill()
             frame_num += 1
 
     def drawPosition(self, context):
+        """Draws the top part of the playhead.
+
+        This should be in sync with the playhead drawn by the timeline.
+        See Timeline.__draw_playhead().
+        """
+        height = self.pixbuf.get_height()
+
+        semi_width = 4
+        semi_height = int(semi_width * 1.61803)
+        y = int(3 * height / 4)
+
         # Add 0.5 so that the line center is at the middle of the pixel,
         # without this the line appears blurry.
         xpos = self.nsToPixel(self.position) - self.pixbuf_offset + 0.5
-        context.set_line_width(PLAYHEAD_WIDTH + 2)
-        set_cairo_color(context, (255, 0, 0))
-        context.move_to(xpos, 0)
-        context.line_to(xpos, context.get_target().get_height())
+        set_cairo_color(context, PLAYHEAD_COLOR)
+
+        context.set_line_width(PLAYHEAD_WIDTH)
+        context.move_to(xpos, y)
+        context.line_to(xpos, height)
+        context.stroke()
+
+        context.set_line_width(PLAYHEAD_WIDTH * 2)
+        context.move_to(xpos, y)
+        context.line_to(xpos + semi_width, y - semi_height)
+        context.line_to(xpos, y - semi_height * 2)
+        context.line_to(xpos - semi_width, y - semi_height)
+        context.close_path()
         context.stroke()
