@@ -1,7 +1,5 @@
+# -*- coding: utf-8 -*-
 # Pitivi video editor
-#
-#       pitivi/utils/validate.py
-#
 # Copyright (c) 2014, Thibault Saunier <thibault.saunier@collabora.com>
 #
 # This program is free software; you can redistribute it and/or
@@ -22,14 +20,14 @@ import locale
 import subprocess
 
 import gi
-from gi.repository import Gtk
-from gi.repository import Gst
-from gi.repository import GES
 from gi.repository import Gdk
+from gi.repository import GES
 from gi.repository import GLib
+from gi.repository import Gst
+from gi.repository import Gtk
 
-from pitivi.utils import ui
 from pitivi.utils import timeline as timelineUtils
+from pitivi.utils import ui
 
 
 CAT = "validate"
@@ -71,6 +69,8 @@ if GstValidate:
 
             if GstValidate:
                 try:
+                    import gi
+                    gi.require_version('Wnck', '3.0')
                     from gi.repository import Wnck
                     Wnck.Screen.get_default().connect("window-opened", self._windowOpenedCb)
                 except ImportError:
@@ -139,6 +139,36 @@ def stop(scenario, action):
     return 1
 
 
+def positionChangedCb(pipeline, position, scenario, action,
+                      wanted_position):
+    if pipeline._waiting_for_async_done:
+        return
+
+    if pipeline._next_seek:
+        return
+
+    print(str(wanted_position), str(position))
+    if wanted_position != position:
+        scenario.report_simple(GLib.quark_from_string(
+            "scenario::execution-error"),
+            "Position after seek (%s) does not match wanted "
+            "one %s" % (Gst.TIME_ARGS(position),
+                        Gst.TIME_ARGS(wanted_position)))
+
+    pipeline.disconnect_by_func(positionChangedCb)
+    action.set_done()
+
+
+def seek(scenario, action):
+    res, wanted_position = GstValidate.utils_get_clocktime(action.structure,
+                                                      "start")
+    scenario.pipeline.simple_seek(wanted_position)
+    scenario.pipeline.connect("position", positionChangedCb, scenario,
+                              action, wanted_position)
+
+    return GstValidate.ActionReturn.ASYNC
+
+
 def set_state(scenario, action):
     wanted_state = action.structure["state"]
     if wanted_state is None:
@@ -475,6 +505,11 @@ def init():
                                          "Pitivi override for the stop action",
                                          GstValidate.ActionTypeFlags.NONE)
 
+        GstValidate.register_action_type("seek", "pitivi",
+                                         seek, None,
+                                         "Pitivi override for the seek action",
+                                         GstValidate.ActionTypeFlags.NONE)
+
         GstValidate.register_action_type("pause", "pitivi",
                                          set_state, None,
                                          "Pitivi override for the pause action",
